from typing import Any
from django.http.response import HttpResponse as HttpResponse
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse
from django.views import generic

from webapp.forms import ArticleForm, CommentForm
from webapp.models import Article


class ArticleDetailView(generic.DetailView):
    pk_url_kwarg = 'id'
    model = Article
    template_name = 'articles/detail.html'
    
    def get_context_data(self, **kwargs):
        comments = self.object.comments.order_by('-created_at')
        return super().get_context_data(
            **kwargs, 
            comments=comments,
            comments_form=CommentForm,
        )


class ArticleCreateView(generic.CreateView):
    model = Article
    template_name = 'articles/article.html'
    form_class = ArticleForm

    def get_success_url(self):
        return reverse('articles_detail', kwargs={'id': self.object.id})


class ArticleUpdateView(generic.FormView):
    template_name = 'articles/article.html'
    form_class = ArticleForm
    
    def dispatch(self, request, *args, **kwargs):
        self.article = self.get_object()
        return super().dispatch(request, *args, **kwargs)
    
    def get_object(self):
        return get_object_or_404(Article, id=self.kwargs.get('id'))

    def get_context_data(self, **kwargs):
        return super().get_context_data(
            **kwargs, 
            article=self.article,
            btn_text='Update',
        )
    
    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['instance'] = self.article
        return kwargs

    def form_valid(self, form):
        self.article = form.save()
        return super().form_valid(form)
    
    def get_success_url(self):
        return reverse('articles_detail', kwargs={'id': self.article.id})


def article_delete_view(request, id):
    get_object_or_404(Article, id=id).delete()
    
    return redirect('index')
